% This script is used in data construction for 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
%  This is step 2 of 3 of the creation of working file from raw data.
%  input data: "raw daily data.xls", "temporary_data.mat"
%  output Matlab data: "temporary_data.mat"
%  
% August 2018

clear

% import Matlab .mat data
load temporary_data;
yr_month_all = yr_month;

% designate the input Excel file
infile='../../data/raw_data/raw daily data.xls';

% Collateralized call rate
% source: Nikkei
% the first res. maintenance period is October 1978 b/c the first
% daily obs. is Oct. 1, 1978
sheetname   = 'collateralized (Nikkei)';
[~,z]       = xlsread(infile,sheetname,'a2:a12419');
temp        = datevec(z);
year        = temp(:,1);
month       = temp(:,2);
day         = temp(:,3);
z           = xlsread(infile,sheetname,'b2:b12419');
z(1)=NaN;z(end)=NaN;  % remove the marker

% take average of daily rates over res. maint. period
j          = find(day==16);
first_yr_month=100*year(j(1))+month(j(1));
fprintf('The first reserve maintenence month is: %1.0f.\n',first_yr_month);
n          = numel(j);
temp=find(day(j(end):end)==15);
if numel(temp)==0
    n      = n-1;
end
z_ave      = [];
yr_month   = [];
for i=1:n
    j15=find(day(j(i):end)==15);
    if numel(j15)==0
        disp('You screwed up!!!');
    end
    j_start= j(i);
    j_end  = j(i)+j15(1)-1;
    if day(j_start)~=16 || day(j_end)~=15
        disp('You screwed up!!!');
    end
        
    temp     = z(j_start:j_end);
    z_ave    = [z_ave;nanmean(temp)];
    yr_month = [yr_month;100*year(j(i))+month(j(i))];
end


% Uncollateralized call rate
% source: Nikkei
% the first res. maintenance period is Aug. 1985 b/c the first
% daily obs. is July 31, 1985
sheetname   = 'uncollateralized (Nikkei)';
[~,z]       = xlsread(infile,sheetname,'a2:a10032');
temp        = datevec(z);
year        = temp(:,1);
month       = temp(:,2);
day         = temp(:,3);
z           = xlsread(infile,sheetname,'b2:b10032');

% take average of daily rates over res. maint. period
j           = find(day==16);
first_yr_month=100*year(j(1))+month(j(1));
fprintf('The first reserve maintenence month is: %1.0f.\n',first_yr_month);
n           = numel(j);
temp        = find(day(j(end):end)==15);
if numel(temp)==0
    n       = n-1;
end
z_ave       = [];
yr_month    = [];
for i = 1:n
    j15     = find(day(j(i):end)==15);
    if numel(j15)==0
        disp('You screwed up!!!');
    end
    j_start = j(i);
    j_end   = j(i)+j15(1)-1;
    if day(j_start)~=16 || day(j_end)~=15
        disp('You screwed up!!!');
    end
        
    temp    = z(j_start:j_end);
    z_ave   = [z_ave;nanmean(temp)];
    yr_month=[yr_month;100*year(j(i))+month(j(i))];
end

% define "uncollateralized_r';
uncollateralized_r = NaN*yr_month_all;
j                  = find(yr_month_all==first_yr_month);
uncollateralized_r(j:j+numel(z_ave)-1)=z_ave;
r_ave=uncollateralized_r;
        
% create "temporary_data.mat" afresh
yr_month=yr_month_all;
save('temporary_data',...
    'yr_month','r_bar',...
    'M','M_req','core_CPI_NSA_ctax',...
    'r_ave'...
);

