% This script is used in data construction for 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
%  This is step 2 of 3 of the creation of working file from raw data.
%  input data: "raw quarterly and monthly output.xls", "temporary_data.mat"
%  output file: "working file.xls"
%  user-defined function: "allocate_between_months_f.m", "merge.m", "HPfilter.m"
%
%  August 2018

clear

% import Matlab .mat data: the only purpose is to retrieve "yr_month"
load temporary_data;
yr_month_all = yr_month;  % spans 197001 - 201312

% Read from "quarterly and monthly series.xls"
infile       = '../../data/raw_data/raw quarterly and monthly output';

% write to "monthly data.xls"
outfile      = 'working file.xls';

% METI all industry index, SA
METI_all_index=xlsread(infile,'all industry index','l10:l309');

yr_month     = [];
for year=1988:2012
    for month=1:12
        yr_month=cat(1,yr_month,year*100+month);
    end
end
% concatenate all-industry index over its old index
%yr_month=cat(1,yr_month_old,yr_month);
%METI_all_index=cat(1,METI_all_index_old,METI_all_index);
z=[yr_month METI_all_index];
% z has
% column 1: year and month (e.g., 201209)
% column 2: METI all industry index

% embed "METI_all_index" on "yr_month_all"
temp           = merge(yr_month_all,z);
METI_all_index = temp(:,4);
clear temp z METI_all_index_old yr_month_old;
        
% read quarterly GDP
% define overall year-quarter
yr_quarter_all=[];
for year=1980:2013
    for quarter=1:4
        yr_quarter_all=cat(1,yr_quarter_all,year*100+quarter);
    end
end
% read chained seasonally adjusted GDP, benchmark year = 2000
quarterly_GDP_old=xlsread(infile,'Cabinet Office 1','b8:b67');
            % from 1980:Q1 - 2010:Q4
yr_quarter=[];
for year=1980:1994
    for quarter=1:4
        yr_quarter=cat(1,yr_quarter,year*100+quarter);
    end
end
z=[yr_quarter quarterly_GDP_old];
% z has
% column 1: year and quarter (e.g., 200301)
% column 2: real GDP, benchmark = 2000

% embed "quarterly_GDP_old" on "yr_quarter_all"
temp                = merge(yr_quarter_all,z);
yr_quarter_all      = temp(:,1);
quarterly_GDP_old   = temp(:,4);
clear temp z yr_quarter;


% read current GDP series
quarterly_GDP=xlsread(infile,'Cabinet Office 2','b8:b87');
    % Cabinet Office, quarterly GDP, SA, base=2005, 1994:Q1- 2013:Q4
yr_quarter=[];
for year=1994:2013
    for quarter=1:4
        yr_quarter=cat(1,yr_quarter,year*100+quarter);
    end
end
z=[yr_quarter quarterly_GDP];
% z has
% column 1: year and quarter (e.g., 200301)
% column 2: real GDP, benchmark = 2005

% embed "quarterly_GDP" on "yr_quarter_all"
temp           = merge(yr_quarter_all,z);
yr_quarter_all = temp(:,1);
quarterly_GDP  = temp(:,4);
clear temp z yr_quarter;


% combine old and current GDP series
j              = find(yr_quarter_all==199401);
ratio          = quarterly_GDP(j)/quarterly_GDP_old(j);
temp           = quarterly_GDP_old(1:j)*ratio;
quarterly_GDP  = cat(1,temp,quarterly_GDP(j+1:end));
clear quarterly_GDP_old

% calcualte monthly real GDP for 198801-201212
t_start_q = find(yr_quarter_all==198801);
t_end_q   = find(yr_quarter_all==201204);
t_start_m = find(yr_month_all==198801);
t_end_m   = find(yr_month_all==201212);
Y         = quarterly_GDP(t_start_q:t_end_q);
x         = METI_all_index(t_start_m:t_end_m);
ta=2;sc=3;type=0;opC=0;rl=[];
res       = chowlin_W(Y,x,ta,sc,type,opC,rl);
interpolated_monthly_GDP=res.y;
yr_month_temp=[];
for year = 1988:2012
    for month=1:12
        yr_month_temp=cat(1,yr_month_temp,year*100+month);
    end
end
z=[yr_month_temp interpolated_monthly_GDP];
% z has
% column 1: year and month (e.g., 200308), from 198801 to 201212
% column 2: interpolated monthly GDP

% embed "interpolated_monthly_GDP" on "yr_month_all"
temp=merge(yr_month_all,z);
yr_month_all             = temp(:,1);
interpolated_monthly_GDP = temp(:,4);

% calculate monthly potential GDP
% obtain quarterly GDP gap
GDP_gap=xlsread(infile,'Cabinet Office 3','d2:d135');
    % Cabinet Office, GDP gap, 1980:Q1- 2013:Q2
temp=[];
for year=1980:2013
    for quarter=1:4
        temp=cat(1,temp,year*100+quarter);
    end
end
temp = temp(1:end-2);  % drop 2013:Q3 and Q4
z    = [temp GDP_gap];

% z has
% column 1: year and quarter (e.g., 200301)
% column 2: GDP gap

% embed "GDP_gap" on "yr_quarter_all"
temp           = merge(yr_quarter_all,z);
yr_quarter_all = temp(:,1);
GDP_gap        = temp(:,4);
clear temp z;

% calculate implied quarterly potential GDP using CAO data
quarterly_potential_GDP    = quarterly_GDP./(1+GDP_gap/100);

% calculate monthly potential GDP using CAO data
t_start_q     = find(yr_quarter_all==198001);
t_end_q       = find(yr_quarter_all==201204);
fprintf('potential GDP available from %1.0f to %1.0f.\n',...
    yr_quarter_all(t_start_q),yr_quarter_all(t_end_q));
t_start_m     = find(yr_month_all==198001);
t_end_m       = find(yr_month_all==201212);
y             = quarterly_potential_GDP(t_start_q:t_end_q);
x             = 1:1:(t_end_q-t_start_q)+1;
xx            = 2/3:1/3:(t_end_q-t_start_q)+1+1/3;
interpolated_monthly_potential_GDP=spline(x,y,xx); 
interpolated_monthly_potential_GDP=...
    [NaN(t_start_m-1,1);...
     interpolated_monthly_potential_GDP';...
     NaN(numel(yr_month_all)-t_end_m,1)];

% calculate HP filtered quarterly potential GDP using CAO data
quarterly_potential_GDP_HP = exp(HPfilter(log(quarterly_GDP(t_start_q:t_end_q)),1600));
y                          = quarterly_potential_GDP_HP;
interpolated_monthly_potential_GDP_HP=spline(x,y,xx); 
                    %or use pchip(x,y,xi);plot(xi,yi,'k-',xi,yi2,'r-')
interpolated_monthly_potential_GDP_HP=...
    [NaN(t_start_m-1,1);...
     interpolated_monthly_potential_GDP_HP';...
     NaN(numel(yr_month_all)-t_end_m,1)];


% write to Excel file
yr_month      = yr_month_all;
header={...
    'yr_month',...
    'r_bar','r_ave',...
    'M','M_req','core_CPI_NSA_ctax',...
    'interpolated_monthly_GDP','interpolated_monthly_potential_GDP',...
    'interpolated_monthly_potential_GDP_HP',...
    };
z=[...
    yr_month,...
    r_bar,r_ave,...
    M,M_req,...
    core_CPI_NSA_ctax,...
    interpolated_monthly_GDP,interpolated_monthly_potential_GDP,...
    interpolated_monthly_potential_GDP_HP,...
    ];
xlswrite(outfile,NaN(size(z,1)+5,size(z,2)+5),'Sheet1','a1');
xlswrite(outfile,header,'Sheet1','a1');
xlswrite(outfile,z,'Sheet1','a2');
