% This script is used in data construction for 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
%  input data: "working file.xls", "somusho19872012.xlsx"
%  output file: "data.xls"
%  August 2018

clear 

[NUM,TXT]  = xlsread('working file.xls');
yr_month   = NUM(:,1);
r_bar      = NUM(:,2);
r          = NUM(:,3);
m          = log(NUM(:,4)./NUM(:,5));
CAO_x      = 100*(log(NUM(:,7))-log(NUM(:,8)));
CAO_trend_growth=[nan(12,1);100*log(NUM(13:end,8)./NUM(1:end-12,8))];
HP_x       = 100*(log(NUM(:,7))-log(NUM(:,9)));
HP_trend_growth=[nan(12,1);100*log(NUM(13:end,9)./NUM(1:end-12,9))];

% m (see data appendix)
m(yr_month>=200607 & yr_month<=200808)=0;
m(yr_month==199912)=mean(m(yr_month>=199903 & yr_month<=200007 & yr_month~=199912));
m(yr_month>=199102 & yr_month<=199104)=0.00070;

% Regime variables (see Section 3 in the text)
s          = ones(length(r),1);s(yr_month>=199903 & yr_month<=200007)=0;s(yr_month>=200103 & yr_month<=200606)=0;s(yr_month>=200812)=0;
qe         = zeros(length(r),1);qe(yr_month>=199903 & yr_month<=200007)=1;qe(yr_month>=200103 & yr_month<=200606)=2;qe(yr_month>=200812)=3;

% r (see data appendix)
r(s==0)=0;r(yr_month>=200812)=0.1;

% import seasonally adjusted CPI series and adjust for the eneryg price swing (see data appendix)
[NUM,TXT]  = xlsread('../../data/processed_data/somusho19872012.xlsx');
core_CPI_SA_ctax=[nan(find(yr_month==198612),1);NUM(:,end);nan(12,1)];
z          = xlsread('../../data/raw_data/raw monthly data.xls','data','i456:i475');
temp0=core_CPI_SA_ctax(find(yr_month==200710));
for i=1:(find(yr_month==201212)-find(yr_month==200710))
    if yr_month(find(yr_month==200710)+i)<200906
        temp1=temp0*z(i+1,end)/z(i,end);
    else
        temp1=temp0*core_CPI_SA_ctax(find(yr_month==200710)+i)/core_CPI_SA_ctax(find(yr_month==200710)+i-1);
    end
    temp0      = temp1;
    tempvec(i) = temp0;
end
core_CPI_SA_ctax(find(yr_month==200711):end-12)=tempvec;

p          = [nan;1200*log(core_CPI_SA_ctax(2:end)./core_CPI_SA_ctax(1:end-1))];
p12        = [nan(12,1);100*log(core_CPI_SA_ctax(13:end)./core_CPI_SA_ctax(1:end-12))];

header     = {'yr_month','r','r_bar', 'm', 'CAO_x', 'CAO_trend_growth','p','p12','s','qe','HP_x','HP_trend_growth'};
z          = [yr_month,r,r_bar,m,CAO_x,CAO_trend_growth,p,p12,s,qe,HP_x,HP_trend_growth];
z          = z(1:end-12,:);
xlswrite('data',NaN(size(z,1)+5,size(z,2)+5),'1970-2012','a1');
xlswrite('data',header,'1970-2012','a1');
xlswrite('data',z,'1970-2012','a2');
