% This script replicates the simple statistics presented in Table 2 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
%  input data: "data.xls"
%  output file: "198801_201212.mat"
%  August 2018


clear 

%% Load data
[NUM,TXT,RAW]=xlsread('data.xls','1970-2012'); 
% Allocate column numbers 
yr_month_num=1;r_num=2;rbar_num=3;m_num=4;cao_x_num=5; cao_trend_num=6;p_num=7;p12_num=8;s_num=9;qe_num=10;hp_x_num=11;hp_trend_num=12;md_num=13;


% Variables
yr_month = NUM(:,yr_month_num); % yyyymm
p        = NUM(:,p_num);        % monthly inflation (annualized rate in %)
p12      = NUM(:,p12_num);      % 12-month inflation (in %)
cao_x    = NUM(:,cao_x_num);      % output cao_x
r        = NUM(:,r_num);        % policy rate 
rbar     = NUM(:,rbar_num);     % effective lower bound
m        = NUM(:,m_num);        % excess reserve rate
trend    = NUM(:,cao_trend_num);    % 12-month potential GDP growth
s        = NUM(:,s_num);        % s
qe       = NUM(:,qe_num);       % qe
hp_x     = NUM(:,hp_x_num);     % hp
hp_trend = NUM(:,hp_trend_num);    % 12-month potential GDP growth

%% Set the sample period (1988.1-2012.12) and subsamples for each regime, and create the corresponding lagged variables
yr_month= yr_month(1+(1988-1970)*12:end);
plag    = p(1+(1988-1970)*12-1:end-1);     % lagged p
p12lag   = p12(1+(1988-1970)*12-1:end-1);  % lagged p12
cao_xlag  = cao_x(1+(1988-1970)*12-1:end-1);   % lagged x
hp_xlag  = hp_x(1+(1988-1970)*12-1:end-1);   % lagged hp_x
rlag    = r(1+(1988-1970)*12-1:end-1);     % lagged r
rbarlag = rbar(1+(1988-1970)*12-1:end-1);  % lagged rbar
mlag    = m(1+(1988-1970)*12-1:end-1);     % lagged m
slag    = s(1+(1988-1970)*12-1:end-1);     % lagged s
qelag   = qe(1+(1988-1970)*12-1:end-1);    % lagged qe
r       = r(1+(1988-1970)*12:end);
p       = p(1+(1988-1970)*12:end);
rbar    = rbar(1+(1988-1970)*12:end);
s       = s(1+(1988-1970)*12:end);
qe      = qe(1+(1988-1970)*12:end);
m       = m(1+(1988-1970)*12:end);
p12     = p12(1+(1988-1970)*12:end);
cao_x   = cao_x(1+(1988-1970)*12:end);
hp_x    = hp_x(1+(1988-1970)*12:end);
trend   = trend(1+(1988-1970)*12:end);
hp_trend   = hp_trend(1+(1988-1970)*12:end);

% P, Z regimes 
paiP=p12(s==1);paiZ=p12(s==0);cao_xP=cao_x(s==1);cao_xZ=cao_x(s==0);rP=r(s==1);rZ=r(s==0);mP=m(s==1);mZ=m(s==0);
% QE1, QE2, QE3 regimes
paiqe1=p12(qe==1);paiqe2=p12(qe==2);paiqe3=p12(qe==3);cao_xqe1=cao_x(qe==1);cao_xqe2=cao_x(qe==2);cao_xqe3=cao_x(qe==3);rqe1=r(qe==1);rqe2=r(qe==2);rqe3=r(qe==3);mqe1=m(qe==1);mqe2=m(qe==2);mqe3=m(qe==3);


%% Table 3
disp(['    Table 2      ']);
disp(['    Subsample P      ']);
disp(['     p12        x        r      m    ']);
disp([[mean(paiP);std(paiP)],[mean(cao_xP);std(cao_xP)], [mean(rP);std(rP)], [mean(mP);std(mP)]])

disp(['    First Zero-Rate Spell (QE1)    ']);
disp(['     p12        x        r      m    ']);
disp([[mean(paiqe1);std(paiqe1)], [mean(cao_xqe1);std(cao_xqe1)], [mean(rqe1);std(rqe1)], [mean(mqe1);std(mqe1)]])

disp(['    Second Zero-Rate Spell (QE2)     ']);
disp(['    p12        x        r      m    ']);
disp([[mean(paiqe2);std(paiqe2)], [mean(cao_xqe2);std(cao_xqe2)], [mean(rqe2);std(rqe2)], [mean(mqe2);std(mqe2)]])

disp(['    Third Zero-Rate Spell  (QE3)    ']);
disp(['    p12        x        r      m    ']);
disp([ [mean(paiqe3);std(paiqe3)], [mean(cao_xqe3);std(cao_xqe3)], [mean(rqe3);std(rqe3)],[mean(mqe3);std(mqe3)]])

%% Creating data files in .mat
save ../../data/subsamples/198801_201212.mat yr_month p p12 cao_x r rbar m  p12lag plag cao_xlag rlag rbarlag slag qelag mlag trend s qe hp_x hp_xlag hp_trend 