% This script replicates the Taylor rule estimates presented in Table 3 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)

clear 

theta0=[0  0.95 0.1 0.05 5 5 6 0] % the estimates are robust to these initial guess values
    
options = optimset('Display','iter','Diagnostics','on');
[X,fval,exitflag,output,grad,hessian]=fminunc('Lfunc_LB',theta0,options);
exitflag
output

alpha_r       = X(1);
gamma_r       = X(2);
beta_r        = X(3:4)';
inv_sig_r     = X(5);
pibar         = X(6);
inv_sig_pibar = X(7);
trendcoeff    = X(end);

% For the later use
load ../data/subsamples/199201_201212.mat 
y             = r(qe==0,1);
AvarB         = inv(hessian/numel(y));
ThetahatB     = X';
T_B           = length(y);
inv_hess_B    =inv(hessian);

% Re-name coefficients
sig_r         = 1/inv_sig_r;
sig_pibar     = 1/inv_sig_pibar;
rho_r         = 1-gamma_r;
alpha_r_star  = alpha_r/rho_r;
beta_r_star   = beta_r/rho_r;
trend_star    = trendcoeff/rho_r;

% Apply the delta method (see p.95 in Hayashi's (2000) econometrics textbook)
Amatrix= [1/(1-gamma_r),alpha_r/(1-gamma_r)^2,zeros(1,6);...
    0,-1,zeros(1,6);...
    0,beta_r(1)/rho_r^2,1/(1-gamma_r),0,zeros(1,4);...
    0,beta_r(2)/rho_r^2,0,1/(1-gamma_r),zeros(1,4);...
    0,0,0,0,-1/inv_sig_r^2,zeros(1,3);...
    0,0,0,0,0,1,0,0;...
    0,0,0,0,0,0,-1/inv_sig_pibar^2,0;
    0,trendcoeff/rho_r^2,  zeros(1,5) ,1/(1-gamma_r)];
stderror=sqrt(diag(Amatrix*AvarB*Amatrix'/numel(y)));

disp(['Estimated coefficients (with std errors in the 2nd row)']);
disp(['   trend    inflation  output speed_of_adj  sig_r    pibar   sig_pibar']);
disp([ [trend_star;stderror(end)],...
     [beta_r_star'; stderror(3:4)'],...
     [rho_r; stderror(2)],...
     [sig_r; stderror(5)  ],...
     [pibar; stderror(6)],...
     [sig_pibar;stderror(7)]]);
disp(['Estimated coefficients (with t-values in the 2nd row)']);
disp(['   trend    inflation  output speed_of_adj  sig_r    pibar   sig_pibar']);
disp([     [trend_star;trend_star/stderror(end)],...
     [beta_r_star'; beta_r_star'./stderror(3:4)'],...
     [rho_r; rho_r/stderror(2)],...
     [sig_r; sig_r/stderror(5)  ],...
     [pibar; pibar/stderror(6)],...
     [sig_pibar;sig_pibar/stderror(7)]]);

save LB_est   AvarB  T_B  ThetahatB    alpha_r  beta_r   gamma_r   pibar  sig_pibar  sig_r   trendcoeff  inv_hess_B  
