% This script replicates the estimates for the excess reserve supply equation presented in Table 4 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
clear 
load ../data/subsamples/198801_201212.mat

y              = m(qe>1);
RHS            = [ones(numel(y),1) p12(qe>1) cao_x(qe>1) mlag(qe>1)];
b              = inv(RHS'*RHS)*RHS'*y;
temp           = y-RHS*b;

alpha_m_qe23   = b(1);
beta_m_qe23    = b(2:end)';
sig_m_qe23     = sqrt(var(temp)*(length(y)-1)/length(y));
std_error_qe23 = [sqrt([diag(var(temp)*inv(RHS'*RHS))']),sqrt(temp'*temp/(length(y)))/sqrt(2*length(y))];

alpha_ms       = alpha_m_qe23;
beta_ms        = beta_m_qe23;
sig_ms         = sig_m_qe23;

save LC_est alpha_ms beta_ms sig_ms

disp(['Estimated coefficients (with std errors in the 2nd row)']); 
disp([' QE23 ']);
disp([' alpha_m     beta_m                        sig_m ']);
disp([ [alpha_m_qe23; std_error_qe23(1)],  [beta_m_qe23; [std_error_qe23(2:4)] ], [sig_m_qe23; std_error_qe23(end)]]);

disp(['Estimated coefficients (with t-values in the 2nd row)']);
disp([' QE23 ']);
disp([' alpha_m     beta_m                        sig_m ']);
disp([ [alpha_m_qe23; alpha_m_qe23/std_error_qe23(1)],  [beta_m_qe23; beta_m_qe23./std_error_qe23(2:4) ], [sig_m_qe23; sig_m_qe23/std_error_qe23(end)]]);