% This script estimates the active excess reserve demand equation discussed in Section 7 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)

clear

load ../data/subsamples/Subsample_demand.mat yr_month r rbar md p12 cao_x  mdlag 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% initial guesses %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% r equation, s=1;
y=md(md>=0.005);
RHS=[ones(numel(y),1) p12(md>=0.005) cao_x(md>=0.005)  r(md>=0.005)-rbar(md>=0.005)  mdlag(md>=0.005) ];

b=inv(RHS'*RHS)*RHS'*y;
alpha_init = b(1);
beta_init  = b(2:4);
gamma_init = b(5);
temp=y-RHS*b;
sig_m_init=sqrt(var(temp,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% Max Likelihood %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
options = optimset('Display','iter','Diagnostics','on');
theta0=[alpha_init beta_init' gamma_init  1/sig_m_init ];

[theta,fval,exitflag,output,grad,hessian]=fminunc('Lfunc_LC_demand',theta0,options);
exitflag
output
disp([theta0' theta']);

T=length(md);
Avar=inv(hessian/T);
se=diag(sqrt(inv(hessian)));% equal to diag(sqrt(Avar/T));

% transformed data
alpha_md     = theta(1);
beta_md      = theta(2:4);
gamma_md     = theta(5); %gamma=1-rho
inv_sig_m    = theta(6);

sig_md = 1/inv_sig_m;


temp=y-RHS*theta(1:5)';
std_dev=sqrt(var(temp,1));

Amatrix=[1,0,0,0,0,0;
         0,1,0,0,0,0;
         0,0,1,0,0,0;
         0,0,0,1,0,0;
         0,0,0,0,1,0;
         0,0,0,0,0,-1/inv_sig_m^2];

newse=sqrt(diag(Amatrix*Avar*Amatrix'/T));

me = ones(T,1)*alpha_md+[p12 cao_x (r-rbar)]*beta_md'+mdlag*gamma_md;
mean_me=mean(me(md>=0.005))

disp(['std errors']);
disp([' alpha_md     beta_md                   gamma_md     sig_md ']);
disp([ [alpha_md; newse(1)],  [ [beta_md gamma_md]; newse(2:end-1)' ], [sig_md; newse(end)]]);

disp(['t-value']);
disp([' alpha_md     beta_md                   gamma_md     sig_md ']);
disp([ [alpha_md; alpha_md/newse(1)],  [ [beta_md gamma_md]; [beta_md gamma_md]./newse(2:end-1)' ], [sig_md; sig_md./newse(end)]]);

save LC_est_demand alpha_md gamma_md beta_md sig_md

