% This script computes impulse responses discussed in Section 7 (active excess reserve demand) in
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
clear 

load ../data/198801_201212.mat
load LA_est_demand  S2_P c2_P Phi2_P
load ../estimates/LA_est    S2_Z c2_Z Phi2_Z
load ../estimates/LB_est
load ../estimates/LC_est
load LC_est_demand


Y       = [p cao_x r m ];
horizon = 60; % the simulation horizon in months


% simulation assumptions
t= find(yr_month==200606);
trend=[trend(1:t); ones(200,1)*trend(t)];
rbar=[rbar(1:t);ones(200,1)*rbar(t)];

for iter_til_10000 =1:10000
    iter_til_10000
    s0_b= 0;
    r0_b = rbar(t);
    m0_b = m(t); 
    s0_a = 1;
    r0_a = rbar(t);
    m0_a = 0; 
    Y0_b =[Y(t,1:2) r0_b m0_b]';
    Y0_a =[Y(t,1:2) r0_a m0_a]';
    plag0_vec_b=p(t-11:t) ;
    plag0_vec_a=plag0_vec_b;
    plag0_vec_b_init=plag0_vec_b;
    plag0_vec_a_init=plag0_vec_a;
    s0_b_init=s0_b;
    s0_a_init=s0_a;
    Y0_b_init=Y0_b;
    Y0_a_init=Y0_a;
    
    [temp_a2,temp_b2,temp_a,temp_b ] =Func_IR_demand(Y,trend,rbar,yr_month,...
    t,horizon,S2_P,S2_Z,c2_P,c2_Z,Phi2_P,Phi2_Z,...
    alpha_r,trendcoeff,beta_r,gamma_r,sig_r,pibar,sig_pibar,...
    alpha_md,beta_md,gamma_md,sig_md,alpha_ms,beta_ms,sig_ms,Y0_b, Y0_a, s0_b, s0_a, plag0_vec_a, plag0_vec_b);
    
    pmat_temp_a(iter,:)=[Y0_a_init(1) temp_a(1,:)];
    gmat_temp_a(iter,:) =[Y0_a_init(2) temp_a(2,:)];
    rmat_temp_a(iter,:) =[Y0_a_init(3) temp_a(3,:)];
    mmat_temp_a(iter,:) =[Y0_a_init(4) temp_a(4,:)];
    clear temp_a
    pmat_temp_b(iter,:)=[Y0_b_init(1) temp_b(1,:)];
    gmat_temp_b(iter,:) =[Y0_b_init(2) temp_b(2,:)];
    rmat_temp_b(iter,:) =[Y0_b_init(3) temp_b(3,:)];
    mmat_temp_b(iter,:) =[Y0_b_init(4) temp_b(4,:)];
    clear temp_b
end

temp1_a=mean(pmat_temp_a,1);
temp1_b=mean(pmat_temp_b,1);
clear pmat_temp_a pmat_temp_b
temp2_a=mean(xmat_temp_a,1);
temp2_b=mean(xmat_temp_b,1);
clear xmat_temp_a xmat_temp_b
temp3_a=mean(rmat_temp_a,1);
temp3_b=mean(rmat_temp_b,1);
clear rmat_temp_a rmat_temp_b
temp4_a=mean(mmat_temp_a,1);
temp4_b=mean(mmat_temp_b,1);
clear mmat_temp_a mmat_temp_b
p_a_mean=temp1_a;
x_a_mean=temp2_a;
r_a_mean=temp3_a;
m_a_mean=temp4_a;
p_b_mean=temp1_b;
x_b_mean=temp2_b;
r_b_mean=temp3_b;
m_b_mean=temp4_b;


clear temp1_a temp2_a temp3_a temp4_a temp1_b temp1_b temp1_b temp1_b

simul_til_400=0;
for simul_til_400=1:400 % # of simulated error bands
    simul_til_400
    load AvarA_est_demand ThetahatA_P
    load ../asydist/AvarA_est ThetahatA_Z 
    load ../estimates/LA_est T_Z K_Z invXX_Z
    load LA_est_demand T_P K_P invXX_P 
    
    S2_P=[ThetahatA_P(13) 0;ThetahatA_P(14:15)];
    temp=wishrnd(S2_P*S2_P',T_P-K_P)/(T_P-K_P);
    S2_P=chol(temp,'lower');    
    temp=mvnrnd(ThetahatA_P(1:12),kron(temp,invXX_P));
    c2_P=[temp(1);temp(7)];Phi2_P=[temp(2:6);temp(8:12)]; 

    S2_Z=[ThetahatA_Z(13) 0;ThetahatA_Z(14:15)];
    temp=wishrnd(S2_Z*S2_Z',T_Z-K_Z)/(T_Z-K_Z);
    S2_Z=chol(temp,'lower');    
    temp=mvnrnd(ThetahatA_Z(1:12),kron(temp,invXX_Z));
    c2_Z=[temp(1);temp(7)];Phi2_Z=[temp(2:6);temp(8:12)]; 
    
    load ../estimates/LB_est ThetahatB AvarB T_B
    temp=mvnrnd(ThetahatB,AvarB/T_B);
    alpha_r         = temp(1);
    gamma_r         = temp(2); %gamma=1-rho
    beta_r          = temp(3:4)';
    sig_r         = 1/temp(5);
    pibar         = temp(6);
    sig_pibar     = 1/temp(7);
    trendcoeff    = temp(end);
    
    load AvarC_est_demand  ThetahatC AvarC T_C
    temp=mvnrnd(ThetahatC,AvarC/T_C);
    alpha_md=temp(1);beta_md=temp(2:4);gamma_md=temp(5);sig_md=1/temp(6);
    
    load ../asydist/AvarC_est  ThetahatC AvarC T_C
    temp=mvnrnd(ThetahatC,AvarC/T_C);
    alpha_ms=temp(1);beta_ms=temp(2:4);sig_ms=temp(5);

    for iter =1:1000
    s0_b= 0;
    r0_b = rbar(t);
    m0_b = m(t); 
    s0_a = 1;
    r0_a = rbar(t);
    m0_a = 0; 
    Y0_b =[Y(t,1:2) r0_b m0_b]';
    Y0_a =[Y(t,1:2) r0_a m0_a]';
    plag0_vec_b=p(t-11:t) ;
    plag0_vec_a=plag0_vec_b;
    plag0_vec_b_init=plag0_vec_b;
    plag0_vec_a_init=plag0_vec_a;
    s0_b_init=s0_b;
    s0_a_init=s0_a;
    Y0_b_init=Y0_b;
    Y0_a_init=Y0_a;
    
    [temp_a2,temp_b2,temp_a,temp_b] = Func_IR_demand(Y,trend,rbar,yr_month,...
    t,horizon,S2_P,S2_Z,c2_P,c2_Z,Phi2_P,Phi2_Z,...
    alpha_r,trendcoeff,beta_r,gamma_r,sig_r,pibar,sig_pibar,...
    alpha_md,beta_md,gamma_md,sig_md,alpha_ms,beta_ms,sig_ms,Y0_b, Y0_a, s0_b, s0_a, plag0_vec_a, plag0_vec_b);

    pmat_temp_a(iter,:) =[Y0_a_init(1) temp_a(1,:)];
    gmat_temp_a(iter,:) =[Y0_a_init(2) temp_a(2,:)];
    rmat_temp_a(iter,:) =[Y0_a_init(3) temp_a(3,:)];
    mmat_temp_a(iter,:) =[Y0_a_init(4) temp_a(4,:)];
    pmat_temp_b(iter,:) =[Y0_b_init(1) temp_b(1,:)];
    gmat_temp_b(iter,:) =[Y0_b_init(2) temp_b(2,:)];
    rmat_temp_b(iter,:) =[Y0_b_init(3) temp_b(3,:)];
    mmat_temp_b(iter,:) =[Y0_b_init(4) temp_b(4,:)];
    clear temp_a temp_b temp_a2 temp_b2
    end
    
    temp1_a=mean(pmat_temp_a,1);
    temp1_b=mean(pmat_temp_b,1);
    clear pimat_temp_a pimat_temp_b
    temp2_a=mean(gmat_temp_a,1);
    temp2_b=mean(gmat_temp_b,1);
    clear gmat_temp_a gmat_temp_b
    temp3_a=mean(rmat_temp_a,1);
    temp3_b=mean(rmat_temp_b,1);
    clear rmat_temp_a rmat_temp_b
    temp4_a=mean(mmat_temp_a,1);
    temp4_b=mean(mmat_temp_b,1);
    clear mmat_temp_a mmat_temp_b
    p_a=temp1_a;
    x_a=temp2_a;
    r_a=temp3_a;
    m_a=temp4_a;
    p_b=temp1_b;
    x_b=temp2_b;
    r_b=temp3_b;
    m_b=temp4_b;
    
    simul_til_400=simul_til_400+1;
    temp_IR_p(iter_valid,:)=p_a-p_b;
    temp_IR_x(iter_valid,:)=x_a-x_b;
    temp_IR_r(iter_valid,:)=r_a-r_b;
    temp_IR_m(iter_valid,:)=m_a-m_b;
       
end


temp_eb_p=prctile(temp_IR_p,[16 84 5 95 50],1) ;
temp_eb_x=prctile(temp_IR_x,[16 84 5 95 50],1) ;
temp_eb_r=prctile(temp_IR_r,[16 84 5 95 50],1) ;
temp_eb_m=prctile(temp_IR_m,[16 84 5 95 50],1) ;

figure('Name','Active excess demand: exit QE','NumberTitle','off');
subplot(2,2,1)
plot([1:1:horizon+1],p_a_mean(1,1:horizon+1)-p_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_p(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_p(2,1:horizon+1),'b--','LineWidth',1.5);
title('monthly inflation, p ','fontsize',12)
axis([1 horizon -1 1])
subplot(2,2,2)
plot([1:1:horizon+1],x_a_mean(1,1:horizon+1)-x_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_x(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_x(2,1:horizon+1),'b--','LineWidth',1.5);
title('output gap, x','fontsize',12)
axis([1 horizon -3 3])
subplot(2,2,3)
plot([1:1:horizon+1],r_a_mean(1,1:horizon+1)-r_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_r(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_r(2,1:horizon+1),'b--','LineWidth',1.5);
title('policy rate, r','fontsize',12)
axis([1 horizon -1 0.5])
subplot(2,2,4)
plot([1:1:horizon+1],m_a_mean(1,1:horizon+1)-m_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_m(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_m(2,1:horizon+1),'b--','LineWidth',1.5);
title('excess reserve rate, m','fontsize',12)
axis([1 horizon -0.3 0.3])

