% This script estimates the reduced form equations with active excess reserve demand discussed in Section 7 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
clear 

%% s=P subsample 
load ../data/subsamples/198801_201212.mat
T=find(yr_month==199503);
yr_month=yr_month(T:end); r=r(T:end);p=p(T:end);p12(T:end);cao_x=cao_x(T:end);rbar=rbar(T:end); s=s(T:end); 
p12lag=p12lag(T:end); plag=plag(T:end); rlag=rlag(T:end); rbarlag=rbarlag(T:end); slag=slag(T:end); qelag=qelag(T:end); cao_xlag=cao_xlag(T:end);
trend=trend(T:end); qe=qe(T:end); md=m(T:end);mdlag=mlag(T:end); 
Y=[p cao_x ] ;Ylag=[plag cao_xlag rlag mdlag] ;
c2_P =zeros(2,1); Phi2_P=zeros(2,size(Y,2)-1);

Z=Y(qelag==0,:);
RHS=[ones(size(Z,1),1) trend(qelag==0)  Ylag(qelag==0,1:4)  ];
B=inv(RHS'*RHS)*RHS'*Z;
B=B';
c2_P=B(:,1);
Phi2_P=B(:,2:end);
E2_P=Z-RHS*B';
temp=E2_P'*E2_P/(size(E2_P,1)-size(RHS,2));
std_error_P=sqrt([diag(temp(1,1)*inv(RHS'*RHS))';diag(temp(2,2)*inv(RHS'*RHS))']);
temp=chol(temp,'lower');
S2_P=[temp(1,1) 0; temp(2,:)];
X_P=RHS;
T_P=length(Z);
invXX_P=inv(RHS'*RHS);
K_P=size(RHS,2);

disp(['Lagged subsample P (with t-values in the 2nd row)']);
disp(['Inflation equation']);
disp(['      c         g        plag      xlag      rlag      mdlag']);
disp([[c2_P(1); c2_P(1)./std_error_P(1,1)],[Phi2_P(1,:); Phi2_P(1,:)./std_error_P(1,2:end)]]);
disp(['Output equation']);
disp(['      c         g        plag      xlag      rlag      mdlag']);
disp([[c2_P(2); c2_P(2)./std_error_P(2,1)],[Phi2_P(2,:); Phi2_P(2,:)./std_error_P(2,2:end)]]);

%% s=Z subsample
load ../data/subsamples/198801_201212.mat
md=m;mdlag=mlag;
Y=[p cao_x ] ;Ylag=[plag cao_xlag mdlag] ;
c2_Z =zeros(2,1); Phi2_Z=zeros(2,size(Y,2)-1);

Z=Y(qelag==1,:);
RHS=[ones(size(Z,1),1) Ylag(qelag==1,:) ];
B=inv(RHS'*RHS)*RHS'*Z;
B=B';
c2_W=B(:,1);
Phi2_W=B(:,2:end);
E2_W=Z-RHS*B';
temp=E2_W'*E2_W/(size(E2_W,1)-size(RHS,2));
std_error_W=sqrt([diag(temp(1,1)*inv(RHS'*RHS))';diag(temp(2,2)*inv(RHS'*RHS))']);
temp=chol(temp,'lower');
S2_W=[temp(1,1) 0; temp(2,:)];
X_W=RHS;
T_W=length(Z);
invXX_W=inv(RHS'*RHS);
K_W=size(RHS,2);

disp(['Lagged subsample W (with t-values in the 2nd row)']);
disp(['Inflation equation']);
disp(['      c        plag      xlag      mlag']);
disp([[c2_W(1); c2_W(1)/std_error_W(1,1)],[Phi2_W(1,:); Phi2_W(1,:)./std_error_W(1,2:end)]]);
disp(['Output equation']);
disp(['      c        plag      xlag      mlag']);
disp([[c2_W(2); c2_W(2)/std_error_W(2,1)],[Phi2_W(2,:); Phi2_W(2,:)./std_error_W(2,2:end)]]);

save LA_est_demand c2_W Phi2_W c2_P Phi2_P S2_P S2_W invXX_P T_P K_P invXX_W T_W K_W


