% This script estimates the reduced-form equations estimated with HP-filter based output gap discussed in Section 7 in 
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
clear
       
%% s=P subsample 
load ../data/subsamples/198801_201212.mat
T=find(yr_month==199503);
yr_month=yr_month(T:end); p=p(T:end);hp_x=hp_x(T:end);r=r(T:end);plag=plag(T:end);hp_xlag=hp_xlag(T:end); rlag=rlag(T:end); 
s=s(T:end); slag=slag(T:end); qelag=qelag(T:end); qe=qe(T:end); hp_trend=hp_trend(T:end);

Y=[p hp_x] ;Ylag=[plag hp_xlag] ; 
c2_P =zeros(2,1); Phi2_P=zeros(2,size(Y,2)-1);
Z           = Y(qelag==0,1:2);
RHS         = [ones(size(Z,1),1) hp_trend(qelag==0) Ylag(qelag==0,:) rlag(qelag==0)  ];
B           = inv(RHS'*RHS)*RHS'*Z;
B           = B';
c2_P        = B(:,1);
Phi2_P      = B(:,2:end);
E2_P        = Z-RHS*B';
temp        = E2_P'*E2_P/(size(E2_P,1)-size(RHS,2));
std_error_P = sqrt([diag(temp(1,1)*inv(RHS'*RHS))';diag(temp(2,2)*inv(RHS'*RHS))']);
temp        = chol(temp,'lower');
S2_P        = [temp(1,1) 0; temp(2,:)];
X_P         = RHS;
T_P         = length(Z);
invXX_P     = inv(RHS'*RHS);
K_P         = size(RHS,2);

disp(['Lagged subsample P (with t-values in the 2nd row)']);
disp(['Inflation equation']);
disp(['      c         g        plag      xlag      rlag']);
disp([[c2_P(1); c2_P(1)./std_error_P(1,1)],[Phi2_P(1,:); Phi2_P(1,:)./std_error_P(1,2:end)]]);
disp(['Output equation']);
disp(['      c         g        plag      xlag      rlag']);
disp([[c2_P(2); c2_P(2)./std_error_P(2,1)],[Phi2_P(2,:); Phi2_P(2,:)./std_error_P(2,2:end)]]);

%% s=Z subsample
load ../data/subsamples/198801_201212.mat
Y=[p hp_x] ;Ylag=[plag hp_xlag ] ;
c2_Z =zeros(2,1); Phi2_Z=zeros(2,size(Y,2)-1);
Z          = Y(qelag>1,1:2);
RHS        = [ones(size(Z,1),1) hp_trend(qelag>1) Ylag(qelag>1,:) rlag(qelag>1,:) mlag(qelag>1,:)];
B          = inv(RHS'*RHS)*RHS'*Z;
B          = B';
c2_QE23    = B(:,1);
Phi2_QE23  = B(:,2:end);
E2_QE23    = Z-RHS*B';
temp       = E2_QE23'*E2_QE23/(size(E2_QE23,1)-size(RHS,2));
std_error_QE23=sqrt([diag(temp(1,1)*inv(RHS'*RHS))';diag(temp(2,2)*inv(RHS'*RHS))']);
temp       = chol(temp,'lower');
S2_Z       = [temp(1,1) 0; temp(2,:)];
c2_Z       = c2_QE23;
Phi2_Z     = Phi2_QE23;
X_Z        = RHS;
T_Z        = length(Z);
invXX_Z    = inv(RHS'*RHS);
K_Z        = size(RHS,2);

disp(['Lagged subsample Z (with t-values in the 2nd row)']);
disp(['Inflation equation']);
disp(['      c         g        plag      xlag      rlag      mlag']);
disp([[c2_QE23(1); c2_QE23(1)/std_error_QE23(1,1)],[Phi2_QE23(1,:); Phi2_QE23(1,:)./std_error_QE23(1,2:end)]]);
disp(['Output equation']);
disp(['      c         g        plag      xlag      rlag      mlag']);
disp([[c2_QE23(2); c2_QE23(2)/std_error_QE23(2,1)],[Phi2_QE23(2,:); Phi2_QE23(2,:)./std_error_QE23(2,2:end)]]);


save LA_est c2_Z Phi2_Z c2_P Phi2_P  S2_P S2_Z T_P invXX_P T_Z invXX_Z K_P K_Z