% This script computes impulse responses reported in Figure 6 in
% "Exiting from QE" by Fumio Hayashi and Junko Koeda (Quantitative Economics forthcoming)
clear 

load ../data/subsamples/198801_201212.mat
load LA_est_notrend.mat
load LB_est_notrend.mat
load ../estimates/LC_est.mat


Y       = [p cao_x r m ];
horizon = 60;

%% Assumptions for the alternative scenario 
r_shock = -1; % the size of policy rae shock in percent
t       = find(yr_month==199305);% the base period
rbar    = [rbar(1:t);  ones(200,1)*rbar(t)];

%% Compute impulse responses
for iter_til_10000 =1:10000
    iter_til_10000
    temp_taylor_b  = r(t);
    temp_taylor_a  = r(t)+ r_shock;
    shock0=randn(2,1);
    %% scenario a in the initial period
    if slag(t)==1         
        if temp_taylor_a >rbar(t) 
            s0_a=1;   
            r0_a = temp_taylor_a;
            m0_a = 0; 
        else
            s0_a=0;
            r0_a = rbar(t);
            m0_a = max(0,alpha_ms+beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    if slag(t)==0        
        if temp_taylor_a >rbar(t) & p12(t) > pibar+sig_pibar*shock0(2)
            s0_a=1;
            r0_a= temp_taylor_a;
            m0_a = 0; 
        else
            s0_a=0;
            r0_a = rbar(t);
            m0_a = max(0,alpha_ms+beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1));  
        end
    end
    %% scenario b in the initial period
    if slag(t)==1         
        if temp_taylor_b >rbar(t) 
            s0_b=1;
            r0_b = temp_taylor_b;
            m0_b = 0; 
        else
            s0_b=0;
            r0_b = rbar(t);
            m0_b = max(0,alpha_ms  +beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    if slag(t)==0         
        if temp_taylor_b >rbar(t) & p12(t) > pibar+sig_pibar*shock0(2)
            s0_b=1;
            r0_b= temp_taylor_b;
            m0_b = 0; 
        else
            s0_b=0;
            r0_b = rbar(t);
            m0_b = max(0,alpha_ms  +beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    Y0_b =[Y(t,1:2) r0_b m0_b]';
    plag0_vec_b=p(t-11:t) ;
    Y0_a =[Y(t,1:2) r0_a m0_a]';
    plag0_vec_a=plag0_vec_b;
    Y0_b_init=Y0_b;
    Y0_a_init=Y0_a;
      
    [temp_a2,temp_b2,temp_a,temp_b] = Func_IR_notrend(Y,rbar,yr_month,...
    t,horizon,S2_P,S2_Z,c2_P,c2_Z,Phi2_P,Phi2_Z,...
    alpha_r,beta_r,gamma_r,sig_r,pibar,sig_pibar,...
    alpha_ms,beta_ms,sig_ms,Y0_b, Y0_a, s0_b, s0_a, plag0_vec_a, plag0_vec_b);

    pmat_temp_a(iter_til_10000,:) =[Y0_a_init(1) temp_a(1,:)];
    xmat_temp_a(iter_til_10000,:) =[Y0_a_init(2) temp_a(2,:)];
    rmat_temp_a(iter_til_10000,:) =[Y0_a_init(3) temp_a(3,:)];
    mmat_temp_a(iter_til_10000,:) =[Y0_a_init(4) temp_a(4,:)];
    pmat_temp_b(iter_til_10000,:) =[Y0_b_init(1) temp_b(1,:)];
    xmat_temp_b(iter_til_10000,:) =[Y0_b_init(2) temp_b(2,:)];
    rmat_temp_b(iter_til_10000,:) =[Y0_b_init(3) temp_b(3,:)];
    mmat_temp_b(iter_til_10000,:) =[Y0_b_init(4) temp_b(4,:)];
    clear temp_a temp_b temp_a2 temp_b2
end

temp1_a=mean(pmat_temp_a,1);
temp1_b=mean(pmat_temp_b,1);
clear pmat_temp_a pmat_temp_b
temp2_a=mean(xmat_temp_a,1);
temp2_b=mean(xmat_temp_b,1);
clear xmat_temp_a xmat_temp_b
temp3_a=mean(rmat_temp_a,1);
temp3_b=mean(rmat_temp_b,1);
clear rmat_temp_a rmat_temp_b
temp4_a=mean(mmat_temp_a,1);
temp4_b=mean(mmat_temp_b,1);
clear mmat_temp_a mmat_temp_b
p_a_mean=temp1_a;
x_a_mean=temp2_a;
r_a_mean=temp3_a;
m_a_mean=temp4_a;
p_b_mean=temp1_b;
x_b_mean=temp2_b;
r_b_mean=temp3_b;
m_b_mean=temp4_b;
clear temp1_a temp2_a temp3_a temp4_a temp1_b temp1_b temp1_b temp1_b

%% Compute error bands
simul_til_400=0;
for simul_til_400=1:400 % # of simulated error bands
    simul_til_400
    load AvarA_est_notrend.mat ThetahatA_P ThetahatA_Z
    load LA_est_notrend.mat T_P T_Z K_P K_Z invXX_P invXX_Z    
    
    S2_P=[ThetahatA_P(9) 0;ThetahatA_P(10:11)];
    temp=wishrnd(S2_P*S2_P',T_P-K_P)/(T_P-K_P);
    S2_P=chol(temp,'lower');    
    temp=mvnrnd(ThetahatA_P(1:8),kron(temp,invXX_P));
    c2_P=[temp(1);temp(5)];Phi2_P=[temp(2:4);temp(6:8)]; 

    S2_Z=[ThetahatA_Z(11) 0;ThetahatA_Z(12:13)];
    temp=wishrnd(S2_Z*S2_Z',T_Z-K_Z)/(T_Z-K_Z);
    S2_Z=chol(temp,'lower');    
    temp=mvnrnd(ThetahatA_Z(1:10),kron(temp,invXX_Z));
    c2_Z=[temp(1);temp(6)];Phi2_Z=[temp(2:5);temp(7:10)]; 

    load LB_est_notrend.mat ThetahatB AvarB T_B
    temp=mvnrnd(ThetahatB,AvarB/T_B);
    alpha_r         = temp(1);
    gamma_r         = temp(2); %gamma=1-rho
    beta_r          = temp(3:4)';
    sig_r         = 1/temp(5);
    pibar         = temp(6);
    sig_pibar     = 1/temp(7);
    
    load ../asydist/AvarC_est.mat  ThetahatC AvarC T_C
    temp=mvnrnd(ThetahatC,AvarC/T_C);
    alpha_ms=temp(1);beta_ms=temp(2:4);sig_ms=temp(5);
    
    for iter =1:1000
    temp_taylor_b  = r(t);
    temp_taylor_a  = r(t)+ r_shock;
    shock0=randn(2,1);
    %% scenario a in the initial period
    if slag(t)==1         
        if temp_taylor_a >rbar(t) 
            s0_a=1;   
            r0_a = temp_taylor_a;
            m0_a = 0; 
        else
            s0_a=0;
            r0_a = rbar(t);
            m0_a = max(0,alpha_ms+beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    if slag(t)==0        
        if temp_taylor_a >rbar(t) & p12(t) > pibar+sig_pibar*shock0(2);
            s0_a=1;
            r0_a= temp_taylor_a;
            m0_a = 0; 
        else
            s0_a=0;
            r0_a = rbar(t);
            m0_a = max(0,alpha_ms+beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1));  
        end
    end
    %% scenario b in the initial period
    if slag(t)==1         
        if temp_taylor_b >rbar(t) 
            s0_b=1;
            r0_b = temp_taylor_b;
            m0_b = 0; 
        else
            s0_b=0;
            r0_b = rbar(t);
            m0_b = max(0,alpha_ms  +beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    if slag(t)==0         
        if temp_taylor_b >rbar(t) & p12(t) > pibar+sig_pibar*shock0(2)
            s0_b=1;
            r0_b= temp_taylor_b;
            m0_b = 0; 
        else
            s0_b=0;
            r0_b = rbar(t);
            m0_b = max(0,alpha_ms  +beta_ms*[p12(t) cao_x(t) m(t-1) ]'+ sig_ms*shock0(1)); 
        end
    end
    Y0_b =[Y(t,1:2) r0_b m0_b]';
    plag0_vec_b=p(t-11:t) ;
    Y0_a =[Y(t,1:2) r0_a m0_a]';
    plag0_vec_a=plag0_vec_b;
    Y0_b_init=Y0_b;
    Y0_a_init=Y0_a;            
    
    [temp_a2,temp_b2,temp_a,temp_b] = Func_IR_notrend(Y,rbar,yr_month,...
    t,horizon,...
    S2_P,S2_Z,c2_P,c2_Z,Phi2_P,Phi2_Z,...
    alpha_r,beta_r,gamma_r,sig_r,pibar,sig_pibar,...
    alpha_ms,beta_ms,sig_ms,Y0_b, Y0_a, s0_b, s0_a, plag0_vec_a, plag0_vec_b);

    pmat_temp_a(iter,:) =[Y0_a_init(1) temp_a(1,:)];
    xmat_temp_a(iter,:) =[Y0_a_init(2) temp_a(2,:)];
    rmat_temp_a(iter,:) =[Y0_a_init(3) temp_a(3,:)];
    mmat_temp_a(iter,:) =[Y0_a_init(4) temp_a(4,:)];
    pmat_temp_b(iter,:) =[Y0_b_init(1) temp_b(1,:)];
    xmat_temp_b(iter,:) =[Y0_b_init(2) temp_b(2,:)];
    rmat_temp_b(iter,:) =[Y0_b_init(3) temp_b(3,:)];
    mmat_temp_b(iter,:) =[Y0_b_init(4) temp_b(4,:)];
    clear temp_a temp_b temp_a2 temp_b2 Y0_b Y0_a s0_b s0_a plag0_vec_a plag0_vec_b
    end
    
    temp1_a=mean(pmat_temp_a,1);
    temp1_b=mean(pmat_temp_b,1);
    clear pmat_temp_a pmat_temp_b
    temp2_a=mean(xmat_temp_a,1);
    temp2_b=mean(xmat_temp_b,1);
    clear xmat_temp_a xmat_temp_b
    temp3_a=mean(rmat_temp_a,1);
    temp3_b=mean(rmat_temp_b,1);
    clear rmat_temp_a rmat_temp_b
    temp4_a=mean(mmat_temp_a,1);
    temp4_b=mean(mmat_temp_b,1);
    clear mmat_temp_a mmat_temp_b
    p_a=temp1_a;
    x_a=temp2_a;
    r_a=temp3_a;
    m_a=temp4_a;
    p_b=temp1_b;
    x_b=temp2_b;
    r_b=temp3_b;
    m_b=temp4_b;
    
    simul_til_400=simul_til_400+1;
    temp_IR_p(simul_til_400,:)=p_a-p_b;
    temp_IR_x(simul_til_400,:)=x_a-x_b;
    temp_IR_r(simul_til_400,:)=r_a-r_b;
    temp_IR_m(simul_til_400,:)=m_a-m_b;
end

temp_eb_p=prctile(temp_IR_p,[16 84 50],1) ;
temp_eb_x=prctile(temp_IR_x,[16 84 50],1) ;
temp_eb_r=prctile(temp_IR_r,[16 84 50],1) ;
temp_eb_m=prctile(temp_IR_m,[16 84 50],1) ;


figure('Name','Figure 6','NumberTitle','off');
subplot(2,2,1)
plot([1:1:horizon+1],p_a_mean(1,1:horizon+1)-p_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_p(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_p(2,1:horizon+1),'b--','LineWidth',1.5);
title('monthly inflation, p ','fontsize',12)
axis([1 horizon -1 1])
subplot(2,2,2)
plot([1:1:horizon+1],x_a_mean(1,1:horizon+1)-x_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_x(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_x(2,1:horizon+1),'b--','LineWidth',1.5);
title('output gap, x','fontsize',12)
axis([1 horizon -3 3])
subplot(2,2,3)
plot([1:1:horizon+1],r_a_mean(1,1:horizon+1)-r_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_r(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_r(2,1:horizon+1),'b--','LineWidth',1.5);
title('policy rate, r','fontsize',12)
axis([1 horizon -1 0.5])
subplot(2,2,4)
plot([1:1:horizon+1],m_a_mean(1,1:horizon+1)-m_b_mean(1,1:horizon+1),'k-',[1:1:horizon+1],temp_eb_m(1,1:horizon+1),'b--',[1:1:horizon+1],temp_eb_m(2,1:horizon+1),'b--','LineWidth',1.5);
title('excess reserve rate, m','fontsize',12)
axis([1 horizon -1.5 1])
